Archy = LibStub("AceAddon-3.0"):NewAddon("Archy", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "LibSink-2.0");
local ldb = LibStub("LibDataBroker-1.1"):NewDataObject("Archy", {
	type = "data source",
	icon = "Interface\\Icons\\trade_archaeology",
	text = "Archy",
	label = "Archy",
	version = "1.5.1",
	align = "right",
	["X-Category"] = "Information",
});
local icon = LibStub("LibDBIcon-1.0")
local deformat = LibStub("LibDeformat-3.0")
local QT = LibStub("LibQTip-1.0");
local tooltip;


Archy.DebugMode = false;

function Archy:DebugPrint(...)
	if Archy.DebugMode then
		Archy:Print("DEBUG: " .. ...);
	end
end

Archy.RacesLoaded = false;
Archy.RareItemColorCode = "|cFF0070DD";
Archy.SolvedArtifact = nil;
Archy.TomTomFrame = nil;
Archy.TomTomPoint = nil;
Archy.MiniMapPool = {};
Archy.MaxRaces = ARCHAEOLOGY_MAX_RACES or 12;
Archy.MaxKeystones = ARCHAEOLOGY_MAX_STONES or 4;
Archy.MaxDigsites = 4;
Archy.RefreshOnNextBagUpdate = false;
Archy.DigsiteTextureIndex = 177;
Archy.LastDigsite = nil;
Archy.Races = {}
Archy.Continents = {
        [WORLDMAP_COSMIC_ID] = "Cosmic", -- That constant is -1
        [0] = "World",
        [1] = "Kalimdor",
        [2] = "Eastern Kingdoms",		
        [3] = "Outlands",
        [4] = "Northrend",
        [5] = "Deepholm",
	};

local ACE_OPTS = {
	name = "Archy",
	handler = Archy,
	type = "group",
	args = {
		toggle = {
			name = "Show/Hide Archy",
			desc = "Shows (or hides) the Archy displays",
			order = 1,
			guiHidden = true,
			type = "execute",
			func = "ToggleFrames"
		},
		lock = {
			name = "Locked",
			desc = "Locks (or unlocks) the Archys displays for repositioning",
			order = 2,
			type = "toggle",
			get = "GetLockState",
			set = "SetLockState"
		}, 
		reset = {
			name = "Reset Positions",
			desc = "Restores the Archy display positions to defaults",
			order = 3,
			type = "execute",
			func = "ResetFramePositions"			
		},
		refresh = {
			name = "Refresh",
			desc = "Refresh data and displays used and provided by Archy",
			order = 4,
			guiHidden = true,
			type = "execute",
			func = "Refresh"			
		},
		minimap = {
			name = "Show Digsites on Minimap",
			desc = "Show the digsites on your minimap",
			order = 6,
			type = "toggle",
			width = "full",
			get = "GetMinimapDigsiteState",
			set = "SetMinimapDigsiteState"
		},
		minimapIcon = {
			name = "Show minimap button",
			desc = "Toggles showing the minimap button",
			order = 7,
			type = "toggle", 
			width = "full",
			cmdHidden = true,
			get = "GetMinimapButtonState",
			set = "SetMinimapButtonState"
		},
		artifactListByContinent = {
			name = "Filter artifact research list for continent",
			desc = "Show only the races that have fragments on the continent",
			order = 7.5,
			type = "toggle",
			width = "full",
			cmdHidden = true,
			get = "GetFilterArtifactsByContinentState",
			set = "SetFilterArtifactsByContinentState"
		},
		extraStats = {
			name = "Show extra stats",
			desc = "Show extra statistics in the minimap (or ldb) tooltip",
			order = 8,
			type = "toggle",
			width = "full",
			cmdHidden = true,
			get = "GetExtraStatsState",
			set = "SetExtraStatsState"
		},
		artifactReadyPing = {
			name = "Play a sound when an artifact can be created",
			desc = "Play a sound when an artifact can be created",
			order = 8.5,
			type = "toggle",
			cmdHidden = true,
			get = "GetArtifactReadyPingState",
			set = "SetArtifactReadyPingState"
		},
		artifactReadyPingWithKeystones = {
			name = "Include keystones",
			desc = "Include keystones in the calculate for playing the sound",
			order = 8.6,
			type = "toggle",
			cmdHidden = true,
			get = "GetArtifactReadyPingUsesKeystonesState",
			set = "SetArtifactReadyPingUsesKeystonesState"
		},
		tomtom = {
			name = "Enable TomTom Integration",
			desc = "Enable (or disable) Archy sending the nearest waypoint information to TomTom",
			order = 9,
			type = "toggle",
			get = "GetTomTomState",
			set = "SetTomTomState"
		},
		nearest = {
			name = "Nearest Digsite",
			desc = "Announces the nearest digsite",
			order = 7,
			guiHidden = true,
			type = "execute",
			func = "AnnounceNearestDigsite"
		},
		config = {
			name = "Config",
			desc = "Shows the configuration window",
			order = 8,
			guiHidden = true,
			type = "execute",
			func = "ShowConfig"
		}
		
	}
}

if TomTom then
	ACE_OPTS.args.tomtomOptions = {
				type = "group",
				name = "TomTom Options",
				order = 10,
				inline = true,
				args = {
					help = {
						type = "description",
						order = 1,
						name = "These options require TomTom.",
					},
					tomtomHereDistance = {
						order = 2,
						type = "range",
						name = "\"Arrival Distance\"",
						desc = "This setting will control the distance at which the waypoint arrow switches to a downwards arrow, indicating you have arrived at your destination",
						cmdHidden = true,
						min = 0, max = 150, step = 5,
						get = "GetTomTomHereDistanceState",
						set = "SetTomTomHereDistanceState",
					},
					tomtomPing = {
						order = 3,
						type = "toggle",
						name = "Play a sound when arriving at a waypoint",
						desc = "When you 'arrive' at a waypoint (this distance is controlled by the 'Arrival Distance' setting in this group) a sound can be played to indicate this.  You can enable or disable this sound using this setting.",
						width = "double",
						cmdHidden = true,
						get = "GetTomTomPingState",
						set = "SetTomTomPingState",
					},
				},
		};
end

local DB_DEFAULTS = {
	global = {
		digsites = {},
		 
	}
} 

function Archy:ShowArchaeology()
	if IsAddOnLoaded("Blizzard_ArchaeologyUI") then
		ShowUIPanel(ArchaeologyFrame)
		return true
	else
		local loaded, reason = LoadAddOn("Blizzard_ArchaeologyUI")
		if loaded then
			ShowUIPanel(ArchaeologyFrame)
			return true
		else
			Archy:Print("ArchaeologyUI not loaded: ".._G["ADDON_"..reason].." Try opening manually.")
			return false
		end
	end
end

function Archy:SetupAstrolabe()
	local orig = GetCurrentMapAreaID();
	for cid, zlist in ipairs{GetMapContinents()} do
		for zid, mapid in ipairs{GetMapZones(cid)} do
			SetMapZoom(cid, zid)
			local mapid = GetCurrentMapAreaID()
			Archy.mapcz[mapid] = {cid, zid}
		end
	end
	SetMapByID(orig);
	Archy.astrolabe.MinimapUpdateTime = 0.1;
end

function Archy:OnInitialize()
	Archy:DebugPrint("DEBUG: Archy:OnInitialize");
	self.db = LibStub("AceDB-3.0"):New("ArchyDB");
	self.astrolabe = DongleStub("Astrolabe-1.0");
	self.mapcz = {};
	Archy:SetupAstrolabe();
	
	self.myDigsites = {};
	self.myArtifacts = {};
	
	if not self.db.global.continentRaces then
		self.db.global.continentRaces = {
				[1] = { 1, 3, 4, 7, 8 },		-- Dwarf, Fossil, Night Elf, Tol'vir, Troll
				[2] = { 1, 3, 4, 8 },			-- Dwarf, Fossil, Night Elf, Troll
				[3] = { 2, 6 },					-- Draenei, Orc
				[4] = { 4, 5, 8, 9 }			-- Night Elf, Nerubian, Troll, Vrykul
			};
	end
	
	if not self.db.char.digsites then
		self.db.char.digsites = {};
	end
	if not self.db.char.artifacts then
		self.db.char.artifacts = {};
	end
	
	if not self.db.char.blacklist then
		self.db.char.blacklist = {};
	end
	
	if self.db.char.artifactReadyPing == nil then
		self.db.char.artifactReadyPing = false;
	end
	if self.db.char.artifactReadyPingUsesKeystones == nil then
		self.db.char.artifactReadyPingUsesKeystones = false;
	end
	if self.db.char.showExtraStats == nil then
		self.db.char.showExtraStats = false;
	end
	
	if not self.db.char.icon then
		self.db.char.icon = { hide = false };
	end
	if self.db.char.hidden == nil then
		self.db.char.hidden = false;
	end
	if self.db.char.hiddenLists == nil then
		self.db.char.hiddenLists = false;
	end
	if self.db.char.locked == nil then
		self.db.char.locked = false;
	end
	if self.db.char.useTomTom == nil then
		self.db.char.useTomTom = false;
	end
	if self.db.char.filterArtifactsByContinent == nil then
		self.db.char.filterArtifactsByContinent = false;
	end
	if self.db.char.showDigsitesOnMinimap == nil then
		self.db.char.showDigsitesOnMinimap = true;
	end
	
	if self.db.global.digsites == nil then
		self.db.global.digsites = {};
	end
	
	self.db.char.frames = self.db.char.frames or { digsites = { x = 0, y = 0 }, artifacts = { x = 0, y = 0 }};
	self.db.char.frames.digsites = self.db.char.frames.digsites or { x = 0, y = 0 };
	self.db.char.frames.artifacts = self.db.char.frames.artifacts or { x = 0, y = 0 };
	
	
	self.KeystoneIDs = {};
	
	self.frames = {};
	self.frames.digsites = CreateFrame("Frame", "Digsites", UIParent, "ArchyDigsiteListFrame");
	self.frames.artifacts = CreateFrame("Frame", "Artifacts", UIParent, "ArchyArtifactListFrame");
	
	self.frames.digsites:ClearAllPoints();
	self.frames.artifacts:ClearAllPoints();
	self.frames.digsites:SetPoint("CENTER", UIParent, "CENTER", self.db.char.frames.digsites.x, self.db.char.frames.digsites.y);
	self.frames.artifacts:SetPoint("CENTER", UIParent, "CENTER", self.db.char.frames.artifacts.x, self.db.char.frames.artifacts.y);

	icon:Register("Archy", ldb, Archy.db.char.icon)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Archy", ACE_OPTS, { "archy", "arch" });
	self.options = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Archy", "Archy");
	Archy:ScheduleTimer("Refresh", 2);
	
		if TomTomLite then
		Archy:Print("TomTomLite Found");
	end

end


function Archy:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.options);
end

function Archy:GetTomTomHereDistanceState()
	return TomTom.profile.arrow.arrival;
end
function Archy:SetTomTomHereDistanceState(info, value)
	TomTom.profile.arrow.arrival = value;
end
function Archy:GetTomTomPingState()
	return TomTom.profile.arrow.enablePing;
end
function Archy:SetTomTomPingState(info, value)
	TomTom.profile.arrow.enablePing = value;
end

function Archy:GetLockState()
	return self.db.char.locked;
end
function Archy:SetLockState(info, value)
	if value then
		Archy:Lock();
	else
		Archy:Unlock();
	end
end

function Archy:GetFilterArtifactsByContinentState()
	return self.db.char.filterArtifactsByContinent;
end
function Archy:SetFilterArtifactsByContinentState(info, value)
	self.db.char.filterArtifactsByContinent = value;
	Archy:RefreshArtifacts();
end

function Archy:GetArtifactReadyPingState()
	return self.db.char.artifactReadyPing;
end
function Archy:SetArtifactReadyPingState(info, value)
	self.db.char.artifactReadyPing = value;
end
function Archy:GetArtifactReadyPingUsesKeystonesState()
	return self.db.char.artifactReadyPingUsesKeystones;
end
function Archy:SetArtifactReadyPingUsesKeystonesState(info, value)
	self.db.char.artifactReadyPingUsesKeystones = value;
end

function Archy:GetExtraStatsState()
	return self.db.char.showExtraStats;
end
function Archy:SetExtraStatsState(info, value)
	self.db.char.showExtraStats = value;
end

function Archy:GetMinimapButtonState()
	return not self.db.char.icon.hide;
end
function Archy:SetMinimapButtonState(info, value)
	self.db.char.icon.hide = not value;
	if self.db.char.icon.hide then
		icon:Hide("Archy")
	else
		icon:Show("Archy")
	end
end

function Archy:GetMinimapDigsiteState()
	return self.db.char.showDigsitesOnMinimap;
end
function Archy:SetMinimapDigsiteState(info, value)
	self.db.char.showDigsitesOnMinimap = value;
	if self.db.char.showDigsitesOnMinimap then
		Archy:Print("Show digsites on the minimap");
	else
		Archy:Print("Hide digsites on the minimap");
	end
	Archy:RefreshMinimapIcons();
end


function Archy:PopulateRaceList()
	if GetNumArchaeologyRaces() == 0 then
		return;
	end
	local numRaces = GetNumArchaeologyRaces();
	for i=1, Archy.MaxRaces do
		local name, currency, texture, itemID =  GetArchaeologyRaceInfo(i);
		Archy.Races[name] = i;
		Archy.KeystoneIDs["" .. itemID] = itemID;
	end
	Archy.RacesLoaded = true;
end

function Archy:OnEnable()
	Archy:DebugPrint("DEBUG: Archy:OnEnable");
	local name = GetArchaeologyInfo();
	local hasArch, rank, maxRank = Archy:GetArchaeologySkillInfo();
	if not self.hasArch then
		self.db.char.hidden = true;
		Archy:DebugPrint("Does NOT have Archaeology");
	else
		Archy:DebugPrint("Does have Archaeology");
	end
	
	self:RegisterEvent("SKILL_LINES_CHANGED");

	if not self.db.char.hidden and not self.db.char.hiddenLists then
		self.frames.digsites:Show();
		self.frames.artifacts:Show();
	else 
		self.frames.digsites:Hide();
		self.frames.artifacts:Hide();
	end
	
	if self.db.char.useTomTom then
		Archy:EnableTomTom();
	end
end

function Archy:SKILL_LINES_CHANGED()
	Archy:UpdateLDB();
end


function Archy:OnDisable()
	Archy:DebugPrint("DEBUG: Archy:OnDisable");
end

function Archy:Refresh()
	if GetNumArchaeologyRaces() == 0 then
		return;
	end
	Archy:DebugPrint("Refresh");
	Archy:PopulateRaceList();
	Archy:RefreshDigsites();
	Archy:RefreshArtifacts();
end

function Archy:RefreshArtifacts()
	Archy:UpdateArtifacts();
	ArchyArtifactListFrame_Update(self.frames.artifacts);
end

function Archy:RefreshDigsites()
	Archy:UpdateDigsites();
	ArchyDigsiteListFrame_Update(self.frames.digsites);
end

function Archy:ToggleListFrames()
	self.db.char.hiddenLists = not self.db.char.hiddenLists;
	if self.db.char.hiddenLists then
		self.frames.artifacts:Hide();
		self.frames.digsites:Hide();
	else
		self.frames.artifacts:Show();
		self.frames.digsites:Show();
	end
end

function Archy:ToggleFrames()
	self.db.char.hidden = not self.db.char.hidden;
	if self.db.char.hidden then
		self.frames.artifacts:Hide();
		self.frames.digsites:Hide();
		if self.db.char.useTomTom then
			Archy:DisableTomTom();
		end
	else
		self.db.char.hiddenLists = false;
		self.frames.artifacts:Show();
		self.frames.digsites:Show();
		if self.db.char.useTomTom then
			Archy:EnableTomTom();
		end
	end
end

function Archy:AnnounceNearestDigsite()
	Archy:Print("Nearest digsite is: " .. Archy:NearestDigsiteName());
end

function Archy:ResetFramePositions()
	Archy:Print("Frame positions have been reset");
	self.frames.digsites:ClearAllPoints();
	self.frames.digsites:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	self.frames.artifacts:ClearAllPoints();
	self.frames.artifacts:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
end

function Archy:ToggleLock()
	if self.db.char.locked then
		Archy:Unlock();
	else
		Archy:Lock();
	end
end

function Archy:Unlock()
	Archy:Print("Unlocked");
	self.db.char.locked = false;
end

function Archy:Lock()
	Archy:Print("Locked");
	self.db.char.locked = true;
end


function Archy:GetTomTomState()
	return self.db.char.useTomTom;
end
function Archy:SetTomTomState(info, value)
	self.db.char.useTomTom = value;
	if self.db.char.useTomTom then
		Archy:Print("Enabling TomTom integration");
		if not Archy.TomTomFrame then
			Archy:EnableTomTom();
		else
			Archy.TomTomFrame:Show();
		end
	else
		Archy:Print("Disabling TomTom integration");
		Archy:DisableTomTom();
	end
end

function Archy:DisableTomTom()
	if TomTom and TomTom.AddZWaypoint and TomTom.RemoveWaypoint then
		if Archy.TomTomFrame then
			if Archy.TomTomPoint then
				Archy.TomTomPoint = TomTom:RemoveWaypoint(Archy.TomTomPoint);
				Archy.TomTomPoint = nil;
			end
			Archy.TomTomFrame:Hide();
			Archy.TomTomFrame:SetScript("OnUpdate", nil);
			Archy.TomTomFrame = nil;
		end
	end
end

function Archy:EnableTomTom()
	if TomTom and TomTom.AddZWaypoint and TomTom.RemoveWaypoint then
		if not Archy.TomTomFrame then
			local t = 0
			Archy.TomTomFrame = CreateFrame("Frame");
			Archy.TomTomFrame:SetScript("OnUpdate", function(self, elapsed)
				t = t + elapsed
				if t > 3 then	-- this doesn't change very often at all; maybe more than 3 seconds?
					t = 0;
					if Archy.TomTomPoint then
						Archy.TomTomPoint = TomTom:RemoveWaypoint(Archy.TomTomPoint);
					end
					local closest = Archy:NearestDigsite();
					if closest and closest.title and closest.title ~= "" then
						Archy.TomTomPoint = TomTom:AddZWaypoint(closest.c, closest.z, closest.x * 100, closest.y * 100, closest.title, false, false, false, false, false, true);
					end
				end
			end)
		end
		Archy.TomTomFrame:Show();
	else
		Archy:Print("TomTom was not found.");
		self.db.char.useTomTom = false;
	end
end

function Archy:GetMapIDFloor(c, z)
	local mapId = Archy.astrolabe:GetMapID(c, z);
	local floors = Archy.astrolabe:GetNumFloors(mapId);
	local fl = floors == 0 and 0 or 1;
	return mapId, fl;
end

function Archy:UpdateDigsites()
	if not Archy.RacesLoaded then
		return;
	end
	Archy:DebugPrint("Archy:UpdateDigsites");
	
	if self.db.char.useTomTom then
		Archy:EnableTomTom();
	end
	
	for continentID=1,4 do
		if not Archy.myDigsites[continentID] then
			Archy.myDigsites[continentID] = {};
		end
		
		SetMapZoom(continentID);
		local totalPOIs = GetNumMapLandmarks();
		local digsiteNum = 1;
		for index = 1,totalPOIs do
			local worldMapPOIName = "WorldMapFramePOI"..index;
			local worldMapPOI = _G[worldMapPOIName];
			if index <= totalPOIs then
				local name, description, textureIndex, px, py, mapLinkID, inBattleMap, graveyardID, areaID = GetMapLandmarkInfo(index);
				if textureIndex == Archy.DigsiteTextureIndex then
					zoneName, fileName, texPctX, texPctY, texX, texY, scrollX, scrollY = UpdateMapHighlight(px, py);


					local raceID = 0
					if Archy.db.global.digsites[name] then
						raceID = Archy.db.global.digsites[name];
					end

					local raceName, currency, texture, itemID = GetArchaeologyRaceInfo(raceID)

					local md, fd = Archy:GetMapIDFloor(continentID, 0);
					local zoneID = Archy:GetZoneID(zoneName);
					local mz, fz = Archy:GetMapIDFloor(continentID, zoneID);
					
					nx, ny = Archy.astrolabe:TranslateWorldMapPosition(md, fd, px, py, mz, fz);

					Archy.myDigsites[continentID][digsiteNum] = {
						zoneName = zoneName,
						zoneID = zoneID,
						name = name,
						raceName = raceName,
						raceTexture = texture,
						raceID = raceID,
						waypoint = { c = continentID, z = zoneID, m = mz, f = fz, x = nx, y = ny },
						minimapIcon = Archy:CreateMinimapIcon(name),
					};

					digsiteNum = digsiteNum + 1
				end
			end
		end
		SetMapToCurrentZone()
	end
	Archy:RefreshMinimapIcons();
end

function Archy:UpdateArtifacts()
	if not Archy.RacesLoaded then
		return;
	end
	Archy:DebugPrint("Archy:UpdateArtifacts");
	if not Archy.hasArch then
		return;
	end
	
	local numRaces = GetNumArchaeologyRaces();
	local raceFrame;
	for i=1, Archy.MaxRaces do
		if i <= numRaces then
			local raceName, currency, texture, itemID = GetArchaeologyRaceInfo(i);
			local numProjects = GetNumArtifactsByRace(i)
			
			if numProjects and numProjects > 0 then
				local name, description, rarity, icon, spellDescription, numSockets, bgTexture =  GetActiveArtifactByRace(i)
				SetSelectedArtifact(i)
				local base, adjust, totalCost = GetArtifactProgress()

				
				numSockets = numSockets or 0;
				local keystones = 0;
				local keystoneName = nil;
				
				
				if itemID and itemID > 0 then
					local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemID);
					keystoneName = itemName;
					keystones = GetItemCount(itemID);
				end
				
				local artifact = {};
				artifact.raceID = i;
				artifact.raceName = raceName;
				artifact.raceTexture = texture;
				artifact.keystoneItemID = itemID;
				artifact.name = name;
				artifact.description = description;
				artifact.rarity = rarity;
				artifact.icon = icon;
				artifact.spellDescription = spellDescription;
				artifact.numSockets = numSockets;
				artifact.bgTexture = bgTexture;
				artifact.progress = base;
				artifact.keystoneAdjust = adjust;
				artifact.totalCost = totalCost;
				artifact.keystones = keystones;
				artifact.keystoneName = keystoneName;
				artifact.pinged = false;

				if Archy.myArtifacts[i] and Archy.myArtifacts[i].pinged then
					local before = Archy.myArtifacts[i].progress;
					local now = base;
					local beforeTotal = Archy.myArtifacts[i].totalCost;
					local nowTotal = totalCost;
					
					if  now >= nowTotal and before >= beforeTotal and now >= before then
						artifact.pinged = true;
					end
					before = before + Archy:CalculateSocketAdjust(Archy.myArtifacts[i].numSockets, Archy.myArtifacts[i].keystones);
					now = now + Archy:CalculateSocketAdjust(artifact.numSockets, artifact.keystones);
					if  Archy.db.char.artifactReadyPingUsesKeystones and now >= nowTotal and before >= beforeTotal and now >= before then
						artifact.pinged = true;
					end
				end

				Archy.myArtifacts[i] = artifact;
			else
				if Archy.myArtifacts[i] then
					Archy.myArtifacts[i] = nil;
				end
			end
		end
	end
end

function Archy:CalculateSocketAdjust(numSockets, keystones)
	local adjust = 0;
	if numSockets > 0 then
		if keystones >= numSockets then
			adjust = numSockets * 12;
		else
			adjust = keystones * 12;
		end
	end
	return adjust;
end

function Archy:RefreshMinimapIcons()
	Archy:RemoveMinimapIcons();
	Archy:AddMinimapIcons();
end

function Archy:RemoveMinimapIcons()
	for _,icon in ipairs(Archy.MiniMapPool) do
		Archy.astrolabe:RemoveIconFromMinimap(icon);
	end
end

function Archy:AddMinimapIcons()
	if Archy.db.char.showDigsitesOnMinimap then
		if Archy.myDigsites[GetCurrentMapContinent()] then
			for _,digsite in pairs(self.myDigsites[GetCurrentMapContinent()]) do
				Archy.astrolabe:PlaceIconOnMinimap(digsite.minimapIcon, digsite.waypoint.m, digsite.waypoint.f, digsite.waypoint.x, digsite.waypoint.y);
			end
		end
	end
end

function Archy:CreateMinimapIcon(name)
	local minimap = CreateFrame("Button", nil, Minimap);
	minimap:SetSize(14,14);
	minimap:RegisterForClicks("RightButtonUp");
	minimap.name = name;
	minimap.kind = "MinimapIcon";
	
	minimap.icon = minimap:CreateTexture(nil, "OVERLAY");
	minimap.icon:SetTexture("Interface\\Archeology\\Arch-Icon-Marker.blp");
	minimap.icon:SetPoint("CENTER", 0, 0);
	minimap.icon:SetSize(14, 14);
	
	minimap:SetScript("OnEnter", OnMinimapEnter);
	minimap:SetScript("OnLeave", OnMinimapLeave);
	minimap:SetScript("OnClick", OnMinimapClick);
	minimap:SetScript("OnUpdate", OnMinimapUpdate);

	tinsert(Archy.MiniMapPool, minimap);

	return minimap;
end

function OnMinimapEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT");
	GameTooltip:SetText(self.name);
	GameTooltip:Show();
end

function OnMinimapLeave(self)
	GameTooltip:Hide();
end

function OnMinimapClick(self, button)
	if (button ~= "RightButton") then
		return;
	else
		-- 
	end
end

function Archy:NearestDigsiteName(arg)
	local closest = Archy:NearestDigsite(arg);
	return closest.title or "";
end			

function Archy:GetContinentZone(mapID, floorNum)
	local cz = Archy.mapcz[mapID];
	if cz then
		local c, z = unpack(cz);
		return c, z;
	end
	return 0, 0;
end

function Archy:GetCurrentPlayerPosition()
	local m, f, x, y = Archy.astrolabe:GetCurrentPlayerPosition();
	local cz = Archy.mapcz[m];
	if cz then
		local c, z = unpack(cz);
		return m, f, c, z, x, y;
	end
end

function Archy:NearestDigsite(arg)
	local distance
	local closest = { title = "", x = 0, y = 0, z = 0, c = 0, m = 0, f = 0 };
	local m, f, c, z, x, y = Archy:GetCurrentPlayerPosition();
			
	local anySite = false;
	if arg and arg.anySite then
		anySite = true;
	end
	
	if Archy.myDigsites[c] then
		for _,digsite in pairs(Archy.myDigsites[c]) do
			if (not Archy.db.char.blacklist[digsite.name] and not anySite) or (anySite) then
				local dist, xDelta, yDelta = Archy.astrolabe:ComputeDistance(m, f, x, y, digsite.waypoint.m, digsite.waypoint.f, digsite.waypoint.x, digsite.waypoint.y);
				if distance then
					if dist < distance then
						distance = dist;
						closest.title = digsite.name;
						closest.c = digsite.waypoint.c;
						closest.z = digsite.waypoint.z;
						closest.x = digsite.waypoint.x;
						closest.y = digsite.waypoint.y;
						closest.m = digsite.waypoint.m;
						closest.f = digsite.waypoint.f;
					end
				else
					distance = dist;
					closest.title = digsite.name;
					closest.c = digsite.waypoint.c;
					closest.z = digsite.waypoint.z;
					closest.x = digsite.waypoint.x;
					closest.y = digsite.waypoint.y;
					closest.m = digsite.waypoint.m;
					closest.f = digsite.waypoint.f;
				end
			end
		end
	end
	
	return closest;
end

-- Data Broker --
function ldb:OnEnter()
	if not QT then
		return;
	end
	tooltip = QT:Acquire("ArchyTT", 5, "LEFT", "RIGHT", "RIGHT", "RIGHT", "RIGHT");
	tooltip:SetScale(1)
	tooltip:SetAutoHideDelay(0.1, self);
	Archy:DrawTooltip();
	tooltip:EnableMouse();
	tooltip:SmartAnchorTo(self);
	tooltip:UpdateScrolling();
	tooltip:Show();
	
	
end


function Archy:DrawTooltip()
		
	local plusIcon = "|TInterface\\Buttons\\UI-PlusButton-Up:18:18:1:0|t"
	local minusIcon = "|TInterface\\Buttons\\UI-MinusButton-Up:18:18:1:0|t"

	tooltip:Hide();
	tooltip:Clear();
	
	local t = {};
	local numCols = 5;
	t[1] = "LEFT";
	t[2] = "LEFT";
	t[3] = "LEFT";
	t[4] = "LEFT";
	t[5] = "LEFT";
	if Archy.db.char.showExtraStats then
		numCols = numCols + 4;
	end
	
	
	tooltip:SetColumnLayout(numCols, unpack(t));
	
	local lineNum = tooltip:AddHeader("b");
	tooltip:SetCell(lineNum, 1, ORANGE_FONT_COLOR_CODE .. "Archy|r", "CENTER", numCols);
	local colIndex = 0;
	local t = {};
	
	-- Artifacts
	if Archy.hasArch then
		lineNum = tooltip:AddLine(" ");
		
		local hasArch, rank, maxRank = Archy:GetArchaeologySkillInfo();
		local skill = format("%s/%s", rank, maxRank);
		if maxRank < 525 and (maxRank - rank) <= 25 then
			skill = skill .. " - |cFFFF0000Visit a trainer!|r";
		end
		tooltip:SetCell(lineNum, 1, NORMAL_FONT_COLOR_CODE .. "Skill:|r " .. skill, "CENTER", numCols);
		
		lineNum = tooltip:AddLine(" ");
		tooltip:SetCell( lineNum, 1, "|cFFFFFF00Artifacts|r", "LEFT", numCols);
	
		lineNum = tooltip:AddLine("b");
		tooltip:SetCell(lineNum, 1, " ", "LEFT", 1);
		tooltip:SetCell(lineNum, 2, NORMAL_FONT_COLOR_CODE .. "Race|r", "LEFT", 1);
		if Archy.db.char.showExtraStats then
			tooltip:SetCell(lineNum, 3, NORMAL_FONT_COLOR_CODE .. "Artifact|r", "LEFT", 2);
			tooltip:SetCell(lineNum, 6, NORMAL_FONT_COLOR_CODE .. "Progress|r", "CENTER", 2);
			tooltip:SetCell(lineNum, 8, NORMAL_FONT_COLOR_CODE .. "Keystones|r", "CENTER", 2);
		else
			tooltip:SetCell(lineNum, 3, NORMAL_FONT_COLOR_CODE .. "Artifact|r", "LEFT", 1);
			tooltip:SetCell(lineNum, 4, NORMAL_FONT_COLOR_CODE .. "Progress|r", "LEFT", 1);
			tooltip:SetCell(lineNum, 5, NORMAL_FONT_COLOR_CODE .. "Keystones|r", "LEFT", 1);
		end

		for raceID,artifact in pairs(Archy.myArtifacts) do
			lineNum = tooltip:AddLine("b");
			tooltip:SetCell(lineNum, 1, " " .. format("|T%s:18:18:0:1:128:128:4:60:4:60|t", artifact.raceTexture), "LEFT", 1);
			tooltip:SetCell(lineNum, 2, artifact.raceName, "LEFT", 1);
			local artifactName = artifact.name;
			if artifact.rarity == 0 then
				artifactName = GRAY_FONT_COLOR_CODE .. artifactName .. "|r";
			else
				artifactName = Archy.RareItemColorCode .. artifactName .. "|r";
			end
		
			local progress;
			if artifact.progress >= artifact.totalCost then
				progress = "|cFF00FF00" .. artifact.progress .. "|r";
			else
				progress = "|cFFFFFFFF" .. artifact.progress .. "|r";
			end
			progress = progress .. "/|cFFFFFFFF" .. artifact.totalCost .. "|r";

			local keystones = format("|cFFFFFF00%s/%s|r", artifact.keystones, artifact.numSockets);
			if artifact.numSockets == 0 and artifact.keystones == 0 then
				keystones = "";
			end
			if self.db.char.showExtraStats then
				tooltip:SetCell(lineNum, 3, artifactName, "LEFT", 2);
				tooltip:SetCell(lineNum, 6, progress, "CENTER", 2);
				tooltip:SetCell(lineNum, 8, keystones, "CENTER", 2);
			else
				tooltip:SetCell(lineNum, 3, artifactName, "LEFT", 1);
				tooltip:SetCell(lineNum, 4, progress, "CENTER", 1);
				tooltip:SetCell(lineNum, 5, keystones, "CENTER", 1);
			end
		
			if self.db.char.showExtraStats then
			
			end
		end
	
		-- Digsites
		tooltip:AddLine(" ");
		lineNum = tooltip:AddLine(" ");
		tooltip:SetCell( lineNum, 1, "|cFFFFFF00Digsites|r", "LEFT", numCols);

		for continentID,continentDigsites in pairs(Archy.myDigsites) do
			lineNum = tooltip:AddLine("b");
			tooltip:SetCell(lineNum, 1,"  " .. ORANGE_FONT_COLOR_CODE .. Archy.Continents[continentID] .. "|r", "LEFT", numCols);
		
			lineNum = tooltip:AddLine("b");
			tooltip:SetCell(lineNum, 1, " ", "LEFT", 1);
			tooltip:SetCell(lineNum, 2, NORMAL_FONT_COLOR_CODE .. "Fragment|r", "LEFT", 1);
			tooltip:SetCell(lineNum, 3, NORMAL_FONT_COLOR_CODE .. "Digsite|r", "LEFT", 1);
			tooltip:SetCell(lineNum, 4, NORMAL_FONT_COLOR_CODE .. "Zone|r", "LEFT", 2);
			if self.db.char.showExtraStats then
				tooltip:SetCell(lineNum, 6, NORMAL_FONT_COLOR_CODE .. "Surveys|r", "LEFT", 1);
				tooltip:SetCell(lineNum, 7, NORMAL_FONT_COLOR_CODE .. "Looted|r", "LEFT", 1);
				tooltip:SetCell(lineNum, 8, NORMAL_FONT_COLOR_CODE .. "Frags|r", "LEFT", 1);
				tooltip:SetCell(lineNum, 9, NORMAL_FONT_COLOR_CODE .. "Keys|r", "LEFT", 1);
			end
		
			for _, digsite in pairs(continentDigsites) do
				local raceName = digsite.raceName;
				if raceName == "UNKNOWN" then
					raceName = " ";
				end
				lineNum = tooltip:AddLine("b");
				tooltip:SetCell(lineNum, 1, " " .. format("|T%s:18:18:0:1:128:128:4:60:4:60|t", digsite.raceTexture), "LEFT", 1);
				tooltip:SetCell(lineNum, 2, raceName, "LEFT", 1);
				tooltip:SetCell(lineNum, 3, digsite.name, "LEFT", 1);
				tooltip:SetCell(lineNum, 4, digsite.zoneName, "LEFT", 2);
				if self.db.char.showExtraStats then
					if Archy.db.char.digsites[digsite.name] then
						tooltip:SetCell(lineNum, 6, Archy.db.char.digsites[digsite.name].surveys, "CENTER", 1);
						tooltip:SetCell(lineNum, 7, Archy.db.char.digsites[digsite.name].looted, "CENTER", 1);
						tooltip:SetCell(lineNum, 8, Archy.db.char.digsites[digsite.name].fragments, "CENTER", 1);
						tooltip:SetCell(lineNum, 9, Archy.db.char.digsites[digsite.name].keystones, "CENTER", 1);
					end
				end
			end
			tooltip:AddLine(" ");
		end
	else
		lineNum = tooltip:AddLine(" ");
		tooltip:SetCell( lineNum, 1, "Learn Archaeology in your nearest major city!", "CENTER", numCols);
	end
	
	tooltip:AddLine(" ");
	lineNum = tooltip:AddLine("c");
	tooltip:SetCell(lineNum, 1, "|cFF00FF00Left-Click to toggle Archy|r", "LEFT", numCols);
	lineNum = tooltip:AddLine("d");
	tooltip:SetCell(lineNum, 1, "|cFF00FF00Shift Left-Click to toggle Archy's on-screen lists|r", "LEFT", numCols);
	lineNum = tooltip:AddLine("e");
	tooltip:SetCell(lineNum, 1, "|cFF00FF00Right-Click to lock/unlock Archy|r", "LEFT", numCols);
	lineNum = tooltip:AddLine("f");
	tooltip:SetCell(lineNum, 1, "|cFF00FF00Middle-Click to display the Archaeology window|r", "LEFT", numCols);
end

function ldb:OnClick(button)
	if button == "LeftButton" then
		if IsShiftKeyDown() then
			Archy:ToggleListFrames();
		else
			Archy:ToggleFrames();
		end
	elseif button == "RightButton" then
		if Archy.db.char.locked then
			Archy:Unlock();
		else
			Archy:Lock();
		end
	elseif button == "MiddleButton" then
		Archy:ShowArchaeology();
	end
end


function Archy:ParseLootMessage(msg)
  local player = UnitName("player")
  local item, quantity = deformat(msg, LOOT_ITEM_SELF_MULTIPLE)
  if item and quantity then
    return player, item, tonumber(quantity)
  end
  quantity = 1
  item = deformat(msg, LOOT_ITEM_SELF)
  if item then
    return player, item, tonumber(quantity)
  end

  player, item, quantity = deformat(msg, LOOT_ITEM_MULTIPLE)
  if player and item and quantity then
    return player, item, tonumber(quantity)
  end

  quantity = 1
  player, item = deformat(msg, LOOT_ITEM)
  return player, item, tonumber(quantity)
end

function Archy:UpdateLDB()
	local hasArch, rank, maxRank = Archy:GetArchaeologySkillInfo();
	if hasArch then
		ldb.text = format("%s/%s", rank, maxRank);
		if maxRank < 525 and (maxRank - rank) <= 25 then
			ldb.text = "|cFFFF0000" .. "Visit a trainer!";
		end
		
		
	else
		ldb.text = "Learn Archaeology";
		
	end
	
end

function Archy:GetArchaeologySkillInfo()
	local _, _, arch = GetProfessions();
	if arch then
		if Archy then
			Archy.hasArch = true;
		end
		local name, texture, rank, maxRank = GetProfessionInfo(arch);
		return true, rank, maxRank;
	else
		if Archy then
			Archy.hasArch = false;
		end
		return false, nil, nil;
	end
end


function Archy:SolveArtifact(raceID, useKeystones)
	local raceName, currency, texture, itemID = GetArchaeologyRaceInfo(raceID);
	local numProjects = GetNumArtifactsByRace(raceID)
	if numProjects and numProjects > 0 then
		local name, description, rarity, icon, spellDescription, numSockets, bgTexture =  GetActiveArtifactByRace(raceID)
		SetSelectedArtifact(raceID)
		local base, adjust, totalCost = GetArtifactProgress()
		
		if numSockets and numSockets > 0 then
			for socketNum=1,numSockets do
				if useKeystones and not ItemAddedToArtifact(itemID) then
					SocketItemToArtifact();
				elseif (not useKeystones) and ItemAddedToArtifact(itemID) then
					RemoveItemFromArtifact();
				end
			end
		end

		local base, adjust, totalCost = GetArtifactProgress()
		Archy:Print("Trying to solve " .. raceName .. " Artifact: " .. name .. " (Fragments: " .. min(base + adjust) .. " of " .. totalCost .. ")");
						
		if CanSolveArtifact() then
			Archy.SolvedArtifact = { name = name, rarity = rarity, raceID = raceID, raceName = raceName };
			SolveArtifact();
			Archy.RefreshOnNextBagUpdate = true;
		end
	else
		Archy:Print("No artifact available to solve for " .. raceName);
		return;
	end		
end

function Archy:GetZoneID(zoneName)
	for i=1, select("#", GetMapZones(GetCurrentMapContinent())), 1 do
		local zone = select(i, GetMapZones(GetCurrentMapContinent()));
		if zone == zoneName then
			return i;
		end
	end
	return nil;
end
			
function Archy:IsValidContinent(continentID)
	if continentID >= 1 and continentID <= 4 then
		return true
	end
	return false
end